class DatabaseNotFoundError(Exception):
    def __init__(self, message):
        super().__init__(message)


class DatabaseExistsError(Exception):
    def __init__(self, message):
        super().__init__(message)



# Copyright (C) 2022 by Jaedson Silva
# This code is licensed under MIT license (see LICENSE.txt for details)

import json
import os

from typing import Union
from datetime import datetime
from cryptography.fernet import Fernet


class JSONHandler:
    def __init__(
        self,
        enc_key: Union[bytes, str],
        database_local: str
    ):
        self._fernet = Fernet(enc_key)
        self._database_local = database_local

    @staticmethod
    def _save_file(file_content: str, filepath: str) -> None:
        with open(filepath, 'w') as writer:
            writer.write(file_content)

    def exists_database(self, database: str) -> bool:
        database_path = os.path.join(self._database_local, database)
        return os.path.isfile(database_path)

    def encrypt_json(self, obj: dict) -> str:
        dict_str = json.dumps(obj)
        encrypted_dict = self._fernet.encrypt(dict_str.encode())
        return encrypted_dict.decode()

    def decrypt_json(self, encrypted: bytes) -> dict:
        decrypted_json = self._fernet.decrypt(encrypted)
        json_data = json.loads(decrypted_json)
        return json_data

    def create_json_database(self, name: str) -> dict:
        database_path = os.path.join(self._database_local, name)
        created_time = str(datetime.now().replace(microsecond=0))

        database = {
            'name': name,
            'created_at': created_time,
            'updated_at': created_time,
            'items': {}
        }

        encrypted_data = self.encrypt_json(database)
        self._save_file(encrypted_data, database_path)

        return database

    def get_database(self, database: str) -> Union[None, dict]:
        database_path = os.path.join(self._database_local, database)

        try:
            with open(database_path, 'rb') as reader:
                encrypted_data = reader.read()
        except FileNotFoundError:
            database = None
        else:
            database = self.decrypt_json(encrypted_data)

        return database

    def update_database(self, database: str, items: dict):
        database_path = os.path.join(self._database_local, database)

        database = self.get_database(database)
        update_time = datetime.now().replace(microsecond=0)

        database['items'] = items
        database['updated_at'] = str(update_time)

        encrypted_json = self.encrypt_json(database)
        self._save_file(encrypted_json, database_path)


if __name__ == '__main__':
    handler = JSONHandler(Fernet.generate_key(), '../databases-test')
    handler.create_json_database('MyDatabase')
    handler.update_database('MyDatabase', {'users': {'name': 'Jaedson'}})

    print(handler.get_database('MyDatabase'))



# Copyright (C) 2022 by Jaedson Silva
# This code is licensed under MIT license (see LICENSE.txt for details)

from .json_handler import JSONHandler
from .exceptions import DatabaseNotFoundError, DatabaseExistsError

from typing import Union, Any


class CookieDB:
    def __init__(
        self,
        key: str = None,
        database_local: str = None,
        autocommit: bool = False
    ):
        """
        Initializes the **JSONHandler** class and prepares the
        encryption key.

        All changes are saved in a temporary dictionary until
        a commit is made, so if `autocommit` is not enabled,
        use the `CookieDB.commit()` method.

        :param key: Encryption key;
        :param database_local: Database directory;
        :param autocommit: If "True", changes will be saved every
        time a database method is called.
        """

        self._json_handler = None
        self._open_database = None
        self._temp_items = {}

        if not database_local:
            database_local = './'

        if not key:
            key = 't45tc90GyT4f4Qim0xt3BsSsZ5oEEgPbM9VstlGwfdg='

        self._key = key
        self._database_local = database_local
        self._autocommit = autocommit

        self._json_handler = JSONHandler(self._key, self._database_local)

    def _auto_commit(self):
        if self._autocommit:
            self.commit()

    def open(self, database_name: str) -> None:
        """
        Stores the name of the database if it exists,
        otherwise an exception `DatabaseNotFoundError`
        is thrown.

        :param database_name: Database name;
        :return: None.
        """

        database_exists = self._json_handler.exists_database(database_name)

        if not database_exists:
            raise DatabaseNotFoundError(f'Database {database_name} not found.')
        else:
            self._open_database = database_name

    def create_database(self, database_name, if_not_exists: bool = False) -> None:
        """
        Create a database at the location specified
        in **database local** in the `CookieDB`
        class instance.

        :param database_name: Database name;
        :param if_not_exists: If "True", exceptions will
        not be thrown if you are trying to create a
        database that already exists;
        :return: None.
        """

        if not self._json_handler.exists_database(database_name):
            self._json_handler.create_json_database(database_name)
        else:
            if not if_not_exists:
                raise DatabaseExistsError(f'Database {database_name} already exists.')

    def commit(self) -> bool:
        """
        Save changes made to the database.

        :return: Returns "True" if there were changes to commit.
        """

        if self._temp_items is None:
            return False

        self._json_handler.update_database(self._open_database, self._temp_items)
        return False

    def create_item(self, path: Union[str, int], value: Any) -> None:
        """
        Creates an item in the database.
        Each path separated by "/" is a key in the JSON file.

        Values can be of type `str`, `int`,
        `float`, `dict`, `list`, or `tuple`.

        :param path: Item path;
        :param value: Item value;
        :return: None.
        """

        items = self._temp_items
        path_split = path.split('/')

        for c, i in enumerate(path_split):
            if c == (len(path_split) - 1):
                items = items.setdefault(i, value)
            else:
                items = items.setdefault(i, {})

        self._auto_commit()

    def get_item(self, path: Union[str, int]) -> Any:
        """
        Get a database item from the path.

        :param path: Item path;
        :return: Returns the obtained value.
        None if nothing is found.
        """

        path_split = path.split('/')
        item = {}

        database = self._json_handler.get_database(self._open_database)
        database_items = database.get('items')

        for i in path_split:
            if i != '':
                if not item:
                    db_item = database_items.get(i)
                    item = db_item
                else:
                    db_item = item.get(i)
                    item = db_item

        return item



from setuptools import setup
from cookiedb import __version__

with open('DOCS/database/0-1-0.md', 'r') as reader:
    full_doc = reader.read()

setup(
    author='Jaedson Silva',
    author_email='imunknowuser@protonmail.com',
    name='cookiedb',
    description='CookieDB is a noSQL document database.',
    long_description_content_type='text/markdown',
    long_description=full_doc,
    version=__version__,
    license='MIT License',
    packages=['cookiedb', 'cookiedb/database'],
    install_requires=['cryptography'],
    project_urls={
        'Source code': 'https://github.com/jaedsonpys/cookiedb',
        'License': 'https://github.com/jaedsonpys/cookiedb/blob/master/LICENSE',
        'Documentation': 'https://github.com/jaedsonpys/cookiedb/tree/master/DOCS'
    }
)
